const sh = require('shelljs');
const utils = require('../utils');

module.exports = context => {
  utils.debug(`git check: ${context.checkGit}`);
  if (context.checkGit) {
    const result = sh.exec(`git status --porcelain ${context.cwd}`, { silent: true });
    if (result.code === 0) {
      const hasUncommittedChanges = result.stdout.trim().length > 0;
      if (hasUncommittedChanges) {
        utils.info(`You have uncommitted changes in your working directory and we don't want to override them`);
        return Promise.reject(new Error('Git repository containing uncommitted is not supported'));
      }
      utils.debug('git working tree is clean');
      return context;
    }
    utils.info(`Your project is not under git control and we don't want to override your changes`);
    return Promise.reject(new Error('Project without git control is not supported'));
  }
  return context;
};
