const globby = require('globby');
const sh = require('shelljs');
const utils = require('../utils');

module.exports = context => {

  utils.debug(`update of sources: ${context.updateSources}`);

  if (!context.updateSources) {
    return context;
  }

  if (!utils.contextHasChanges(context)) {
    utils.debug('skipping check, cause there were no changes');
    return Promise.resolve(context);
  }

  const changes = context.changes;
  const cwd = context.cwd;

  utils.info('Checking for suspicious sources...');

  return new Promise(resolve => {
    const suspiciousSources = [];
    const names = Object.keys(changes).join('|');

    const files = globby
      .sync(['**/*'], {
        dot: true,
        gitignore: true,
        ignore: utils.IGNORE_GLOBS,
        cwd
      }).join(' ');

    const result = sh.exec(`egrep -HIn -e '(@wix/)?(bower_components/)?(${names})[^-]' ${files} | egrep -v -e '@wix/' -e 'bower_components/'`, { silent: true });

    if (result.code === 0) {
      suspiciousSources.push(...result.stdout.trim().split('\n'));
    }

    resolve(Object.assign(context, {
      suspiciousSources: suspiciousSources.filter(source => source.trim() !== '')
    }));

  });
};
