const chalk = require('chalk');
const fs = require('fs');
const os = require('os');
const path = require('path');
const utils = require('../utils');

module.exports = context => {

  utils.debug('printing report...');

  if (!utils.contextHasChanges(context)) {
    utils.debug('no changes were done, so skipping the report');
    utils.success();
    return false;
  }

  const folder = fs.mkdtempSync(path.join(os.tmpdir(), 'us-'));

  const updatedPackagesFile = path.join(folder, 'packages.txt');
  fs.writeFileSync(updatedPackagesFile, JSON.stringify(context.changes, null, 2));

  const outdatedPackagesFile = path.join(folder, 'packages-outdated.txt');
  fs.writeFileSync(outdatedPackagesFile, JSON.stringify(context.outdated, null, 2));

  const updatedSourcesFile = path.join(folder, 'files.txt');
  fs.writeFileSync(updatedSourcesFile, JSON.stringify(context.changedSources, null, 2));

  const suspiciousSourcesFile = path.join(folder, 'files-suspicious.txt');
  fs.writeFileSync(suspiciousSourcesFile, context.suspiciousSources.join('\n'));

  utils.alert([
    chalk.white(Object.keys(context.changes).length) + ' ' + chalk.green('package(s) updated:'),
    chalk.cyan(updatedPackagesFile),
    chalk.white(Object.keys(context.outdated).length) + ' ' + chalk.green('outdated package(s) found:'),
    chalk.cyan(outdatedPackagesFile),
    chalk.white(context.changedSources.length) + ' ' + chalk.green('source file(s) updated:'),
    chalk.cyan(updatedSourcesFile),
    chalk.white(context.suspiciousSources.length) + ' ' + chalk.green('suspicious usage(s) found, please review them:'),
    chalk.cyan(suspiciousSourcesFile)
  ].join('\n'));

  return true;
};
