const globby = require('globby');
const path = require('path');
const fs = require('fs');
const sh = require('shelljs');
const utils = require('../utils');

const GLOBS = [
  '**/*.+(js|es6|jsx|ts|tsx|sh|sass|scss|md|vm|ejs|html|haml)',
  'package.json',
  '**/tsconfig.json'
];

const WEBPACK_LOADERS = `(?:.*!|~)*`;

const PATTERNS = [
  name => new RegExp(`(\\s+from\\s+["']${WEBPACK_LOADERS})(${name})(\\b)`, 'g'),
  name => new RegExp(`(\\bimport\\s+["']${WEBPACK_LOADERS})(${name})(\\b)`, 'g'),
  name => new RegExp(`(\\brequire\\(["']${WEBPACK_LOADERS})(${name})(\\b)`, 'g'),
  name => new RegExp(`(\\bnode_modules\\/)(${name})(\\b)`, 'g'),
  name => new RegExp(`(\\bdeclare\\s+module\\s+["']*)(${name})(\\b)`, 'g'),
  name => new RegExp(`(\\bunpkg/)(${name})(@)`, 'g')
];

const LOOSE_PATTERNS = PATTERNS.concat([
  name => new RegExp(`(\\b(?:npm|yarn).*\\s+)(${name})(\\b)`, 'g')
]);

module.exports = context => {

  if (!context.updateSources) {
    return context;
  }

  const changes = context.changes;
  const cwd = context.cwd;

  if (!utils.contextHasChanges(context)) {
    return Promise.resolve(context);
  }

  utils.info('Updating sources...');

  return new Promise(resolve => {
    globby
      .sync(GLOBS, {
        gitignore: true,
        ignore: utils.IGNORE_GLOBS,
        cwd
      })
      .forEach((file, index, files) => {
        utils.infoSameLine(`Checking file ${index + 1} out of ${files.length}`);
        utils.debug(`Checking file ${index + 1} out of ${files.length}: ${file}`);
        Object.keys(changes).forEach(oldName => {
          updateSource(cwd, file, oldName, changes[oldName]);
        });
      });

    utils.infoSameLineStop();

    const result = sh.exec('git status --porcelain ' + cwd, { silent: true });
    const changedSources = result.stdout.replace(/^ M /gm, '').trim().split('\n');

    resolve(Object.assign(context, { changedSources }));
  });
};

function updateSource(cwd, file, oldName, newName) {
  const fullFilePath = path.join(cwd, file);
  const patterns = ['.md', '.json'].indexOf(path.extname(file)) !== -1 ? LOOSE_PATTERNS : PATTERNS;

  const contents = patterns.reduce((acc, curr) =>
    acc.replace(curr(oldName), '$1' + newName + '$3'), fs.readFileSync(fullFilePath, 'utf8'));

  fs.writeFileSync(fullFilePath, contents, 'utf8');
}
