const fs = require('fs');
const chalk = require('chalk');
const childProcess = require('child_process');
const debug = require('debug')('update-scopes');

const options = {
  encoding: 'utf8'
};

module.exports.debug = debug;

module.exports.info = message => {
  console.log(message);
};

module.exports.infoSameLine = message => {
  process.stdout.write(`${message}\r`);
};

module.exports.infoSameLineStop = () => {
  process.stdout.write('\n');
};

module.exports.alert = message => {
  console.log(chalk.bgRed.bold('Hey 🤠  your attention is required!'));
  console.log(message);
};

module.exports.success = () => {
  console.log('Your packages are properly scoped 👍');
};

module.exports.parseJson = value => {
  return new Promise((resolve, reject) => {
    try {
      resolve(JSON.parse(value));
    } catch (error) {
      reject(error);
    }
  });
};

module.exports.readJson = name => {
  return new Promise((resolve, reject) => {
    fs.readFile(name, options, (err, data) => {
      err ? reject(err) : resolve(data);
    });
  })
  .then(data => JSON.parse(data));
};

module.exports.writeJson = (name, data) => {
  return new Promise((resolve, reject) => {
    const stringified = JSON.stringify(data, null, 2) + '\n';
    fs.writeFile(name, stringified, options, err => {
      err ? reject(err) : resolve(data);
    });
  });
};

module.exports.npm = params => {
  return new Promise((resolve, reject) => {
    childProcess.exec(['npm', params].join(' '), { maxBuffer: 1024 * 1024 }, (error, stdout) => {
      error ? reject(new Error('npm command failed')) : resolve(stdout);
    });
  }).then(stdout => module.exports.parseJson(stdout));
};

module.exports.run = tasks => {
  return context => {
    const reducer = (acc, curr) => acc.then(context => curr(context));
    return tasks.reduce(reducer, Promise.resolve(context));
  };
};

module.exports.IGNORE_GLOBS = [
  '**/node_modules/**',
  '**/bower_components/**',
  '**/dist/**',
  './package-lock.json',
  '**/.git/**',
  'pom.xml',
  'bower.json',
  'fedops.json',
  '**/maven/**'
];

module.exports.contextHasChanges = context =>
  Object.keys(context.changes).length > 0;

module.exports.isInCi = () =>
  !!(process.env.BUILD_NUMBER || process.env.TEAMCITY_VERSION || process.env.CI);

module.exports.isInWindows = () =>
  /^win/.test(process.platform);

module.exports.isInYarn = () =>
  (/\byarn\b/).test(process.env.npm_config_user_agent || '');
